/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.model;

import se.llbit.chunky.resources.Texture;
import se.llbit.math.AABB;
import se.llbit.math.Quad;
import se.llbit.math.Ray;
import se.llbit.math.Vector3d;
import se.llbit.math.Vector4d;

@SuppressWarnings("javadoc")
public class FenceModel {
	private static AABB post = new AABB(.375, .625, 0, 1, .375, .625);
	
	private static Quad[][] plank = {
		// front
		{
			// -- bottom plank
			
			// left
			new Quad(new Vector3d(.4375, .5625, .4), new Vector3d(.4375, .5625, 0),
					new Vector3d(.4375, .375, .4), new Vector4d(.4, 0, .5625, .375)),
	
			// right
			new Quad(new Vector3d(.5625, .5625, 0), new Vector3d(.5625, .5625, .4),
					new Vector3d(.5625, .375, 0), new Vector4d(0, .4, .5625, .375)),
		
			// top
			new Quad(new Vector3d(.4375, .5625, 0), new Vector3d(.4375, .5625, .4),
					new Vector3d(.5625, .5625, 0), new Vector4d(0, .4, .4375, .5625)),
					
			// bottom
			new Quad(new Vector3d(.5625, .375, 0), new Vector3d(.5625, .375, .4),
					new Vector3d(.4375, .375, 0), new Vector4d(0, .4, .5625, .4375)),
			
			// -- top plank
					
			// left
			new Quad(new Vector3d(.4375, .9375, .4), new Vector3d(.4375, .9375, 0),
					new Vector3d(.4375, .75, .4), new Vector4d(.4, 0, .9375, .75)),
	
			// right
			new Quad(new Vector3d(.5625, .9375, 0), new Vector3d(.5625, .9375, .4),
					new Vector3d(.5625, .75, 0), new Vector4d(0, .4, .9375, .75)),
		
			// top
			new Quad(new Vector3d(.4375, .9375, 0), new Vector3d(.4375, .9375, .4),
					new Vector3d(.5625, .9375, 0), new Vector4d(0, .4, .4375, .5625)),
					
			// bottom
			new Quad(new Vector3d(.5625, .75, 0), new Vector3d(.5625, .75, .4),
					new Vector3d(.4375, .75, 0), new Vector4d(0, .4, .5625, .4375)),
		},
		// back
		{
			// -- bottom plank
			
			// left
			new Quad(new Vector3d(.4375, .5625, 1), new Vector3d(.4375, .5625, .6),
					new Vector3d(.4375, .375, 1), new Vector4d(1, .6, .5625, .375)),
	
			// right
			new Quad(new Vector3d(.5625, .5625, .6), new Vector3d(.5625, .5625, 1),
					new Vector3d(.5625, .375, .6), new Vector4d(.6, 1, .5625, .375)),
		
			// top
			new Quad(new Vector3d(.4375, .5625, .6), new Vector3d(.4375, .5625, 1),
					new Vector3d(.5625, .5625, .6), new Vector4d(.6, 1, .4375, .5625)),
					
			// bottom
			new Quad(new Vector3d(.5625, .375, .6), new Vector3d(.5625, .375, 1),
					new Vector3d(.4375, .375, .6), new Vector4d(.6, 1, .5625, .4375)),
					
			// -- top plank
			
			// left
			new Quad(new Vector3d(.4375, .9375, 1), new Vector3d(.4375, .9375, .6),
					new Vector3d(.4375, .75, 1), new Vector4d(1, .6, .9375, .75)),
	
			// right
			new Quad(new Vector3d(.5625, .9375, .6), new Vector3d(.5625, .9375, 1),
					new Vector3d(.5625, .75, .6), new Vector4d(.6, 1, .9375, .75)),
		
			// top
			new Quad(new Vector3d(.4375, .9375, .6), new Vector3d(.4375, .9375, 1),
					new Vector3d(.5625, .9375, .6), new Vector4d(.6, 1, .4375, .5625)),
					
			// bottom
			new Quad(new Vector3d(.5625, .75, .6), new Vector3d(.5625, .75, 1),
					new Vector3d(.4375, .75, .6), new Vector4d(.6, 1, .5625, .4375)),
		},
	};
	
	private static Quad[][] rotPlank = new Quad[4][];
	
	static {
		rotPlank[0] = plank[0];
		rotPlank[1] = plank[1];
		for (int j = 2; j < 4; ++j) {
			rotPlank[j] = new Quad[plank[j-2].length];
			for (int i = 0; i < plank[j-2].length; ++i) {
				rotPlank[j][i] = plank[j-2][i].getYRotated();
			}
		}
	}

	public static boolean intersect(Ray ray, Texture texture) {
		boolean hit = false;
		ray.t = Double.POSITIVE_INFINITY;
		if (post.intersect(ray)) {
			texture.getColor(ray);
			ray.t = ray.tNear;
			hit = true;
		}
		for (int i = 0; i < 4; ++i) {
			if ((ray.getBlockData() & (1 << i)) != 0) {
				for (Quad quad : rotPlank[i]) {
					if (quad.intersect(ray)) {
						texture.getColor(ray);
						ray.n.set(quad.n);
						ray.t = ray.tNear;
						hit = true;
					}
				}
			}
		}
		if (hit) {
			ray.color.w = 1;
			ray.distance += ray.t;
			ray.x.scaleAdd(ray.t, ray.d, ray.x);
		}
		return hit;
	}
}
